﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using Infragistics.Controls.Schedules;
using System.Collections.Specialized;
using System.Collections.Generic;
using Infragistics;
using System.Diagnostics;

namespace ScheduleSample
{
	public static class ScheduleHelper
	{
		#region DateNavigator

		/// <summary>
		/// DateNavigator Attached Dependency Property
		/// </summary>
		public static readonly DependencyProperty DateNavigatorProperty =
			DependencyProperty.RegisterAttached("DateNavigator", typeof(Calendar), typeof(ScheduleHelper),
				new PropertyMetadata((Calendar)null,
					new PropertyChangedCallback(OnDateNavigatorChanged)));

		/// <summary>
		/// Returns the navigator control that should be updated when the visible dates of the schedule control changes
		/// </summary>
		public static Calendar GetDateNavigator( DependencyObject d )
		{
			return (Calendar)d.GetValue(DateNavigatorProperty);
		}

		/// <summary>
		/// Sets the navigator control that should be updated when the visible dates of the schedule control changes.
		/// </summary>
		public static void SetDateNavigator( DependencyObject d, Calendar value )
		{
			d.SetValue(DateNavigatorProperty, value);
		}

		/// <summary>
		/// Handles changes to the DateNavigator property.
		/// </summary>
		private static void OnDateNavigatorChanged( DependencyObject d, DependencyPropertyChangedEventArgs e )
		{
			ScheduleControlBase ctrl = d as ScheduleControlBase;

			if ( null != ctrl )
			{
				Calendar oldCalendar = e.OldValue as Calendar;
				Calendar newCalendar = e.NewValue as Calendar;

				if (oldCalendar != null)
					GetCalendarSynchronizer(oldCalendar).Control = null;

				if (newCalendar != null)
					GetCalendarSynchronizer(newCalendar).Control = ctrl;
			}
		}
		#endregion // DateNavigator

		#region CalendarSynchronizer

		private static readonly DependencyProperty CalendarSynchronizerProperty =
			DependencyProperty.RegisterAttached("CalendarSynchronizer", typeof(Synchronizer), typeof(ScheduleHelper),
				new PropertyMetadata((Synchronizer)null));

		private static Synchronizer GetCalendarSynchronizer( Calendar calendar )
		{
			Synchronizer sync = (Synchronizer)calendar.GetValue(CalendarSynchronizerProperty);

			if ( sync == null )
			{
				sync = new Synchronizer(calendar);
				calendar.SetValue(CalendarSynchronizerProperty, sync);
			}

			return sync;
		}

		#endregion //CalendarSynchronizer

		#region Synchronizer class
		private class Synchronizer
		{
			#region Member Variables

			private Calendar _calendar;
			private ScheduleControlBase _control;
			private bool _isSynchronizingVisibleDates;

			#endregion // Member Variables

			#region Constructor
			internal Synchronizer( Calendar calendar )
			{
				_calendar = calendar;
				calendar.SelectedDatesChanged += new EventHandler<SelectionChangedEventArgs>(this.OnCalendarSelectedDatesChanged);
			}
			#endregion // Constructor

			#region Properties
			internal ScheduleControlBase Control
			{
				get { return _control; }
				set
				{
					if ( value != _control )
					{
						if ( null != _control )
							_control.VisibleDates.CollectionChanged -= new NotifyCollectionChangedEventHandler(this.OnVisibleDatesChanged);

						_control = value;

						if ( null != _control )
							_control.VisibleDates.CollectionChanged += new NotifyCollectionChangedEventHandler(this.OnVisibleDatesChanged);
					}
				}
			}
			#endregion // Properties

			#region Methods
			private void OnCalendarSelectedDatesChanged( object sender, SelectionChangedEventArgs e )
			{
				this.SynchronizeVisibleDates(true);
			}

			private void OnVisibleDatesChanged( object sender, NotifyCollectionChangedEventArgs e )
			{
				this.SynchronizeVisibleDates(false);
			}

			private void SynchronizeVisibleDates( bool fromCalendar )
			{
				if ( _isSynchronizingVisibleDates )
					return;

				ScheduleControlBase scb = _control;
				System.Windows.Controls.Calendar cal = _calendar;

				if ( scb == null )
					return;

				_isSynchronizingVisibleDates = true;

				try
				{
					if ( fromCalendar )
					{
						scb.VisibleDates.Clear();

						foreach ( DateTime date in cal.SelectedDates )
							scb.VisibleDates.Add(date);
					}
					else
					{
						cal.SelectedDates.Clear();

						if ( scb.VisibleDates.Count > 0 )
						{
							List<DateRange> ranges = new List<DateRange>();
							DateRange range = new DateRange(scb.VisibleDates[0]);

							// this is to workaround some perf issues in the ms calendar class
							for ( int i = 1, count = scb.VisibleDates.Count; i < count; i++ )
							{
								DateTime date = scb.VisibleDates[i];

								if ( date.Subtract(range.End).Days > 1 )
								{
									ranges.Add(range);
									range = new DateRange(date);
								}
								else
									range.End = date;
							}

							ranges.Add(range);

							foreach ( var item in ranges )
								cal.SelectedDates.AddRange(item.Start, item.End);
						}
					}
				}
				catch ( Exception ex )
				{
					Debug.Assert(fromCalendar, "An exception happened when initializing the calendar from the schedule control?");

					if ( fromCalendar )
					{
						MessageBox.Show(string.Format("An error occurred while trying to synchronize the VisibleDates from the Calendar selection.\r\n{0}", ex.Message), "Sync VisibleDates Error", MessageBoxButton.OK);
					}
				}
				finally
				{
					_isSynchronizingVisibleDates = false;
				}

				if ( fromCalendar && cal.SelectedDates.Count != scb.VisibleDates.Count )
					this.SynchronizeVisibleDates(false);
			}
			#endregion // Methods
		} 
		#endregion // Synchronizer class
	}
}
